require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/fairpay2/account/client_account_create'
require 'uu/fairpay2/account/client_account_attributes'
require 'uu/fairpay2/account/client_account_set_attributes'

# Module Unicorn Universe.
module UU
  # module Fairpay2.
  module Fairpay2
    # Module of Account
    module Account
      # Module of Client Account
      # This service allows managing Client Account.
      module ClientAccount
        extend self

        # Service path
        PATH = 'uu/fairpay2/account/ClientAccount'


        # Creates a new Client Account. The command creates a new Client Account in a specified
        # location  with passed DTO parameters.<br>
        #
        # <b>Required attributes:</b><br>
        #   name, meta_artifact_uri, company_uri, account_number, account_currency
        #
        # @param [UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where client account will be created.
        # @param [UU::Fairpay2::Account::ClientAccountCreate] DTO containing attributes of the new Client Account
        #
        # @return [UU::OS::UESURI] UESURI of the new Client Account
        #
        #
        # @example
        #      # Prepare Folder or Organization Unit URI, where we want to create Client Account.
        #      location_uri = UU::OS::UESURI.new('ues:TERRITORY:FOLDER_OR_ORG_UNIT');
        #
        #      # Create Client Account
        #      UU::Fairpay2::Account::ClientAccount.create(location_uri,
        #        :name => "Name",
        #        :meta_artifact_uri => UU::OS::UESURI.new('ues:TERRITORY:META_ARTIFACT_CLIENT_ACCOUNT'),
        #        :account_number => "12345/666",
        #        :account_currency => "CZK",
        #        :company_uri => UU::OS::UESURI.new('ues:TERRITORY:ARTIFACT_COMPANY'));
        def create(location_uri, client_account = nil)
          svc = UU::OS::REST::RemoteClient.new(ClientAccount)
          client_account_create = UU::Fairpay2::Account::ClientAccountCreate.new(client_account).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('create', location_uri, client_account_create)
            return UU::OS::UESURI.new(res)
          end
        end

        # Obtains Client Account attributes from REST service and returns them
        #
        # @param [String,UU::OS::UESURI] client_account_uri UESURI of Client Account.
        #
        # @return [UU::FairPay2::Account::ClientAccountAttributes] Client Account attributes
        #
        #
        # @example
        #      # Prepare Client Account URI, where we want to get attributes.
        #      client_account_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT');
        #
        #      # Get Client Account attributes
        #      UU::Fairpay2::Account::ClientAccount.get_attributes(client_account_uri);
        def get_attributes(client_account_uri)
          svc = UU::OS::REST::RemoteClient.new(ClientAccount)
          UU::OS::QoS::QoSHandler.auto_retry do
            attrs = svc.get('getAttributes', client_account_uri)
            return UU::Fairpay2::Account::ClientAccountAttributes.new(attrs)
          end
        end

        # Sets attributes of Client Account.
        #
        # @param [String,UU::OS::UESURI] client_account_uri UESURI of modified Client Account.
        # @param [UU::Fairpay2::Account::ClientAccountSetAttributes] client_account_set_attributes DTO containing modified attributes of the Client Account.
        #
        # @return [UU::OS::UESURI] UESURI of modified Client Account
        #
        #
        # @example
        #      # Prepare Client Account URI, where we want to set attributes.
        #      client_account_uri = UU::OS::UESURI.new('ues:TERRITORY:CLIENT_ACCOUNT');
        #
        #      # Set Client Account attributes
        #      UU::Fairpay2::Account::ClientAccount.set_attributes(client_account_uri,
        #        :name => "Name");
        def set_attributes(client_account_uri, client_account_set_attributes = nil)
          svc = UU::OS::REST::RemoteClient.new(ClientAccount)
          ca_attrs = UU::Fairpay2::Account::ClientAccountSetAttributes.new(client_account_set_attributes).to_json

          UU::OS::QoS::QoSHandler.auto_retry do
            res = svc.post('setAttributes', client_account_uri, ca_attrs)
            return UU::OS::UESURI.new(res)
          end
        end

      end
    end
  end
end
